# -*- coding: utf-8 -*-

"""
    Configuration file to run PatchPooling + LR classifier for Face PAD
    toward detection of mask attacks in NIR.
"""
#------------------------------------------------------------------------------

import os
sub_directory = "pooling_lr"

# define preprocessor:

from bob.pad.face.preprocessor import FaceCropAlign
from bob.bio.video.preprocessor import Wrapper
from bob.bio.video.utils import FrameSelector
from bob.pad.face.preprocessor.FaceCropAlign import auto_norm_image as _norm_func

# parameters and constants
FACE_SIZE = 128
RGB_OUTPUT_FLAG = False
USE_FACE_ALIGNMENT = True
ALIGNMENT_TYPE = "lightcnn"
MIN_FACE_SIZE = 50
NORMALIZATION_FUNCTION = _norm_func
NORMALIZATION_FUNCTION_KWARGS = {'n_sigma':3.0, 'norm_method':'MAD'}

_image_preprocessor = FaceCropAlign(
    face_size=FACE_SIZE,
    rgb_output_flag=RGB_OUTPUT_FLAG,
    use_face_alignment=USE_FACE_ALIGNMENT,
    min_face_size=MIN_FACE_SIZE,
    alignment_type=ALIGNMENT_TYPE,
    #normalization_function = NORMALIZATION_FUNCTION,
    #normalization_function_kwargs = NORMALIZATION_FUNCTION_KWARGS,
    )

_frame_selector = FrameSelector(selection_style="all")

preprocessor = Wrapper(preprocessor=_image_preprocessor,\
    frame_selector=_frame_selector)

#------------------------------------------------------------------------------

# define extractor:

from bob.paper.nir_patch_pooling.extractor import PatchPoolingCNN
from bob.bio.video.extractor import Wrapper
from bob.extension import rc as _rc

_model_directory = _rc["lightcnn9.model_directory"]
_model_name = "LightCNN_9Layers_checkpoint.pth.tar"
_model_file = os.path.join(_model_directory, _model_name)

if not os.path.exists(_model_file):
    print("Error: Could not find the LightCNN-9 model [{}].\nPlease follow \
    the download instructions from README".format(_model_directory))
    exit(0)

extractor = Wrapper(PatchPoolingCNN(model_file=_model_file),\
    frame_selector=_frame_selector)

#------------------------------------------------------------------------------

# define algorithm

from bob.pad.base.algorithm import LogRegr

C = 1.0

algorithm = LogRegr(C=C, frame_level_scores_flag=True)

#------------------------------------------------------------------------------
